package org.example.test;

import javax.tools.JavaFileManager;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.Arrays;

/**
 * Proxy that transparently delegates to a JavaFileManager impl, but hides the physical
 * implementation of the delegated file manager to prevent ECJ performing analysis based
 * on the superclasses of the file manager object.
 */
public final class DemotingFileManagerProxy implements InvocationHandler {
    private final JavaFileManager fileManager;

    private DemotingFileManagerProxy(JavaFileManager fileManager) {
        this.fileManager = fileManager;
    }

    @Override
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        System.err.printf("Calling %s(%s)%n", method.getName(), Arrays.toString(args));
        try {
            var result = method.invoke(fileManager, args);
            System.err.printf("Call to %s(%s) returned %s%n", method.getName(), Arrays.toString(args), result);
            return result;
        } catch (Throwable ex) {
            ex.printStackTrace();
            throw ex;
        }
    }

    public static JavaFileManager demote(JavaFileManager fileManager) {
        return (JavaFileManager) Proxy.newProxyInstance(
                fileManager.getClass().getClassLoader(),
                new Class<?>[]{ JavaFileManager.class },
                new DemotingFileManagerProxy(fileManager)
        );
    }
}
